angular.module('puller', [])
  .constant('pullerConfig', {
    images: {
      pull: 'img/arrow.png',
      release: 'img/arrow.png',
      refreshing: 'img/loading.gif'
    }
  })
  .directive('puller', ['$rootScope', 'pullerConfig', '$timeout', function($rootScope, pullerConfig, $timeout) {
    return {
      restrict: 'E',
      template:
        '<div class="puller-wrapper">' +
          '<div class="puller-header">' +
            '<img class="puller-image" ng-src="{{image}}"/>' +
          '</div>' +
        '</div>',
      transclude: true,
      scope: {
        parent: '@',
        topCallback: '&',
        bottomCallback: '&',
        label: '@',
        disablePull: '&'
      },
      link: function(scope, element) {
        var texts = pullerConfig.texts;
        var images = pullerConfig.images;
        var pullerHeight = 60;
        var lastY = 0;
        var status = 'pull';
        var pullerWrapper = element.find('.puller-wrapper');
        var pullerHeader = pullerWrapper.find('.puller-header');
        var hitRockBottom = false;

        var currentDisable = scope.disablePull;

        var bottomCall = function() {
          try {
            if (!hitRockBottom) {
              scope.bottomCallback();
              hitRockBottom = true;
              $timeout(function() {
                hitRockBottom = false;
              }, 1000
            );
            }
          } catch (err) {
            console.error(err);
          }
        };

        var pullerGoAway = function() {
          setStatus('pull');
          pullerWrapper.animate({height: 0}, 250);
          currentDisable = scope.disablePull;
        };

        var topCall = function() {
          try {
            scope.topCallback();
            currentDisable = function() {
              return true;
            };
          } catch (err) {
            console.error(err);
          }
          $timeout(pullerGoAway, 2000);
        };

        var scrolling = function(ev) {
          if (currentDisable()) {
            return;
          }
          if(!$(scope.parent).hasClass('active')) {
            return;
          }
          if(lastY == 0) {
            return; //Otherwise this thing can be jumpy
          }
          var currentY = ev.touches ? ev.touches[0].pageY : false;

          //at the top start making puller
          if ((window.scrollY === 0) && (currentY > lastY)) {
            ev.preventDefault();
            if (status != 'refreshing') {
              setStatus('pull');
            }
            var newHeight = pullerWrapper.height();
            newHeight += currentY - lastY;
            newHeight = newHeight >= pullerHeight ? pullerHeight : newHeight;
            pullerWrapper.height(newHeight + 'px');

            if (newHeight == pullerHeight && status != 'refreshing') {
              setStatus('refreshing');
              topCall();
            }
            lastY = currentY;
            return;
          }
          //Puller might have to animate away
          if (pullerWrapper.height() && (window.scrollY == 0) && status != 'refreshing') {
            var newHeight = pullerWrapper.height() + currentY - lastY;
            if (newHeight <= 0) {
              newHeight = 0;
            }
            pullerWrapper.height(newHeight + 'px');
            ev.preventDefault();
          }

          if($(window).scrollTop() >= ($(document).height() - $(window).height())) {
            bottomCall();
          }
          lastY = currentY;
        };

        var startTouch = function(ev) {
          if (scope.disablePull()) {
            return;
          }
          if(!$(scope.parent).hasClass('active')) {
            return;
          }
          lastY = ev.touches[0].pageY; //fix jumpy touchmoves
        };

        var endTouch = function(ev) {
          if (currentDisable()) {
            return;
          }
          if(!$(scope.parent).hasClass('active')) {
            return;
          }
          if (pullerWrapper.height() && pullerWrapper.height() != pullerHeight) {
            pullerGoAway();
          }
        };

        document.addEventListener("touchmove", scrolling, false);
        document.addEventListener("touchstart", startTouch, false);
        document.addEventListener("touchend", endTouch, false);

        var setStatus = function(newState){
          status = newState;
          scope.$apply(function(){
            scope.image = images[status];
          });
        };
      }
    };
  }]);
