MaddenCompanionApp.directive('tabs', ['$rootScope', '$location', function($rootScope, $location) {
  return {
    restrict: 'E',
    transclude: true,
    controller: ['$scope', '$attrs', '$timeout', function($scope, $attrs, $timeout) {
      var panes = $scope.panes = [];
      $scope.elements = [];
      var selectedPaneKey = $attrs.selectedPaneKey || 'selectedPane';
      var selectPromise = false;

      $scope.selectKey = function(key) {
        $scope.select(panes[key]);
      };

      $scope.select = function(pane) {
        if (selectPromise) {
          $timeout.cancel(selectPromise);
        }
        var key;
        angular.forEach(panes, function(pane2, key2) {
              if (pane != pane2) {
                  pane2.selected = false;
              } else {
                key = key2;
              }
        });
        selectPromise = $timeout(function() {
          $scope.isDropOpen = false;
          pane.selected = true;
          $scope.selectedPaneTitle = pane.tabTitle;
          selectedPaneKey = key;
          if (pane.url && $location.path() != pane.url) {
            $location.path(pane.url);
          }
          $rootScope.$emit('tabChanged', pane, key);
          window.scroll(0,0);
          selectPromise = false;
        }, 100);
      };

      this.addPane = function(pane, element) {
        $scope.panes.push(pane);
        $scope.elements.push(element);
      };

      $scope.$watch(selectedPaneKey, function(newTab) {
        angular.forEach(panes, function(pane, key) {
          if (newTab == (pane.tabId || key) && ! pane.selected) {
            $scope.select(pane);
          }
        });
      });

      $scope.$on('$routeChangeSuccess', function() {
        angular.forEach(panes, function(pane) {
          if (pane.url == $location.path()) {
            $scope.select(pane);
          }
        });
      });

    }],
    template:
      '<div class="tabbable">' +
        '<span class="current-nav-tab" ng-class="{active:isDropOpen}" ng-click="isDropOpen = !isDropOpen">[[selectedPaneTitle]]</span>' +
        '<ul class="nav nav-tabs" ng-class="{active:isDropOpen}"><li ng-repeat="pane in panes" ng-class="{active:pane.selected}" class="tab-[[$index]]" ng-click="select(pane)" ng-hide="pane.tabHide"><a class="nav-tab-title [[pane.tabId]]">[[pane.tabTitle]]</a></li></ul>' +
        '<div class="tab-wrapper"><div class="tab-content" ng-transclude></div></div>' +
        '</div>',
    replace: true
  };
}]);

MaddenCompanionApp.directive('pane', function() {
  return {
    require: '^tabs',
    restrict: 'E',
    transclude: true,
    scope: {
      tabTitle: '@',
      url: '=',
      tabHide: '=',
      tabId: '='
    },
    link: function($scope, $element, $attrs, tabsCtrl) {
      tabsCtrl.addPane($scope, $element);
    },
    template: '<div class="tab-pane pane-[[tabTitle|lowercase]]" ng-class="{active: selected}"><div ng-transclude class="tab-pane-content"></div></div>',
    replace: true
  };
});
